<?php

/**
 * @file
 * Contains exerciser.drush.inc.
 */

/**
 * Implements hook_drush_command().
 */
function exerciser_drush_command() {

  $commands['exerciser'] = [
    'description' => 'Runs a test on all nodes of a specified content type',
    'aliases' => ['ex'],
    'arguments' => [
      'nid' => 'nid',
    ],
    'options' => [
      'host' => 'Host',
      'curl' => 'Use curl for connections',
      'curl_timeout' => 'Curl timeout in seconds.',
      'db' => 'Debug',
    ],
    'examples' => [
      'drush x1' => 'exerciser',
      'drush x2' => 'exerciser --host=http://yoursite.com',
      'drush x3' => 'exerciser --curl',
      'drush x4' => 'exerciser --curl --curl_timeout=2',
    ],
  ];

  $commands['exerciser-restart'] = [
    'description' => 'Restarts an exerciser run on all nodes of a specified content type.',
    'aliases' => ['exr'],
    'arguments' => [
      'nid' => 'nid',
    ],
    'options' => [
      'host' => 'Host',
      'curl' => 'Use curl for connections',
      'curl_timeout' => 'Curl timeout in seconds.',
      'db' => 'Debug',
    ],
    'examples' => [
      'drush x1' => 'exerciser-restart',
      'drush x2' => 'exerciser-restart --host=http://yoursite.com',
      'drush x3' => 'exerciser-restart --curl',
      'drush x4' => 'exerciser-restart --curl --curl_timeout=2',
    ],
  ];

  return $commands;
}

/**
 * Implements exerciser (ex) command.
 */
function drush_exerciser($content_types = NULL) {
  $content_type = _drush_choose_content_type('what content type');
  if ($content_type === FALSE) {
    print "\r\nExerciser - Canceled\r\n";
  }

  $run_timestamp = time();

  if ($nid == NULL) {
    _emit_divider_line();
    _emit_header_line();
    _emit_divider_line();

    $node_count = 1;
    $connection = \Drupal::database();
    $query = $connection->query("SELECT `nid`, `type` FROM {node_field_data} WHERE `status` = 1 AND `type` = :type ORDER BY `nid` ASC",
                                [':type' => $content_type]);
    $results = $query->fetchAll();
    foreach ($results as $row) {
      $fields_array = workout_test_node($row->nid, $row->type, $run_timestamp);
      $fields_array['id'] = $node_count;
      _emit_data_line($fields_array);
      $node_count++;
    }
    _emit_divider_line();

  }

  print "\r\n\r\nCompleted - drush exerciser.\r\n\r\n";

}

/**
 * Implements exerciser-restart (exr) command.
 */
function drush_exerciser_restart($content_types = NULL) {
  $content_type = _drush_choose_content_type('what content type');
  if ($content_type === FALSE) {
    print "\r\nExerciser - Canceled\r\n";
  }

  $run_timestamp = _drush_choose_run('To Restart');

  if ($run_timestamp == 0) {
    print "\r\n\r\n Canceled - drush workout-restart $nid.\r\n\r\n";
    return;
  }

  $connection = \Drupal::database();

  if ($nid == NULL) {
    // Find the max(nid) for the run and start there.
    $nid = $connection->query("SELECT MAX(`nid`) FROM {workout} WHERE `run` = :run",
                              [
                                ':run' => $run_timestamp,

                              ])->fetchField();

  }

  if (is_numeric($nid)) {

    _emit_divider_line();
    _emit_header_line();
    _emit_divider_line();

    $query = $connection->query("SELECT `nid`, `type` FROM {node_field_data}  WHERE  `status` = 1  AND `type` = :type AND `nid` >= :nid ORDER BY `nid` ASC",
      [
        ':nid' => $nid,
        ':type' => $content_type,
      ]);
    $results = $query->fetchAll();
    foreach ($results as $row) {
      _emit_data_line(workout_test_node($row->nid, $row->type, $run_timestamp));
    }

    _emit_divider_line();

  }

  print "\r\n\r\n Completed - drush exerciser-restart $nid.\r\n\r\n";

}

/**
 * A choose_content_type helper.
 */
function _drush_choose_content_type($why = '') {
  $options_arr = [];
  $option_number = 1;
  $connection = \Drupal::database();
  $query = $connection->query("SELECT DISTINCT `type` FROM {node} ORDER BY `type` ASC");
  $results = $query->fetchAll();
  foreach ($results as $row) {
    $options_arr[$option_number++] = $row->type;
  }
  $choice = drush_choice($options_arr, "Choose content_type $why.");
  if ($choice == 0) {
    return FALSE;
  }
  else {
    return $options_arr[$choice];
  }
}

/**
 * Alternate choose_content_type helper.
 */
function __drush_choose_content_type($why = '') {
  $options_arr = [];
  $types = \Drupal::entityTypeManager()
    ->getStorage('node_type')
    ->loadMultiple();

  $content_types_array = array_keys($types);
  foreach ($content_types_array as $content_type) {
    $options_arr[$content_type] = $content_type;
  }
  $choice = drush_choice($options_arr, "Choose content_type $why.");

  return $choice;

}
